# -*- coding: utf-8 -*-

from .device import Device
from .const import (
    #LOGGER,
    STATUS_RESPONSE_METERS,
    STATUS_RESPONSE_EMETERS,
    STATUS_RESPONSE_METERS_POWER,
    STATUS_RESPONSE_METERS_TOTAL,
    STATUS_RESPONSE_METERS_VOLTAGE,
    STATUS_RESPONSE_METERS_PF,
    STATUS_RESPONSE_METERS_CURRENT,
    STATUS_RESPONSE_METERS_TOTAL_RETURNED,
    INFO_VALUE_CURRENT_CONSUMPTION,
    INFO_VALUE_TOTAL_CONSUMPTION,
    INFO_VALUE_TOTAL_RETURNED,
    INFO_VALUE_VOLTAGE,
    INFO_VALUE_POWER_FACTOR,
    INFO_VALUE_CURRENT
)

class PowerMeter(Device):
    """Class to represent a power meter value"""
    def __init__(self, block, channel, positions, meters=None, volt_pos=None,
                 pf_pos=None, current_pos=None, voltage_to_block=False,
                 tot_pos=None):
        super(PowerMeter, self).__init__(block)
        self.id = block.id
        if channel > 0:
            self.id += "-" + str(channel)
            self._channel = channel - 1
            self.device_nr = channel
        else:
            self._channel = 0
        if meters is None:
            self.meters = [self._channel]
        else:
            self.meters = meters
        self._positions = positions
        self._volt_pos = volt_pos
        self._pf_pos = pf_pos
        self._tot_pos = tot_pos
        self._current_pos = current_pos
        self._voltage_to_block = voltage_to_block
        self.sensor_values = {}
        self.device_type = "POWERMETER"
        self.info_values = {}
        self.state = None

    def update_status_information(self, status):
        """Update the status information."""
        factor = 1
        if STATUS_RESPONSE_EMETERS in status:
            meters = status.get(STATUS_RESPONSE_EMETERS) #Shelly EM
        else:
            meters = status.get(STATUS_RESPONSE_METERS)
            factor = 60
        if meters:
            power = 0
            total = None
            total_returned = None
            for meterpos in self.meters:
                meter = meters[meterpos]
                if meter.get(STATUS_RESPONSE_METERS_POWER) is not None:
                    power += float(meter.get(STATUS_RESPONSE_METERS_POWER))
                if meter.get(STATUS_RESPONSE_METERS_TOTAL) is not None:
                    total = (total or 0) + \
                        float(meter.get(STATUS_RESPONSE_METERS_TOTAL))
                if meter.get(STATUS_RESPONSE_METERS_TOTAL_RETURNED) is not None:
                    total_returned = (total_returned or 0) + \
                        float(meter.get(STATUS_RESPONSE_METERS_TOTAL_RETURNED))
                if meter.get(STATUS_RESPONSE_METERS_VOLTAGE) is not None:
                    if self._voltage_to_block:
                        self.block.info_values[INFO_VALUE_VOLTAGE] = \
                            float(meter.get(STATUS_RESPONSE_METERS_VOLTAGE))
                    else:
                        self.info_values[INFO_VALUE_VOLTAGE] = \
                            float(meter.get(STATUS_RESPONSE_METERS_VOLTAGE))
                elif status.get(STATUS_RESPONSE_METERS_VOLTAGE) is not None:
                    #Fix for Shelly 2.5 etc
                    self.block.info_values[INFO_VALUE_VOLTAGE] = \
                        float(status.get(STATUS_RESPONSE_METERS_VOLTAGE))
                if meter.get(STATUS_RESPONSE_METERS_PF) is not None:
                    self.info_values[INFO_VALUE_POWER_FACTOR] = \
                        float(meter.get(STATUS_RESPONSE_METERS_PF))
                if meter.get(STATUS_RESPONSE_METERS_CURRENT) is not None:
                    self.info_values[INFO_VALUE_CURRENT] = \
                        float(meter.get(STATUS_RESPONSE_METERS_CURRENT))
            self.state = power
            if total_returned is not None:
                self.info_values[INFO_VALUE_TOTAL_RETURNED] \
                    = round(total_returned / factor)
            if total is not None:
                self.info_values[INFO_VALUE_TOTAL_CONSUMPTION] \
                    = round(total / factor)
            self._update(self.state, info_values=self.info_values)

    def update(self, data):
        """Get the power"""
        update = False
        if self._positions:
            self.state = sum(data.get(pos, 0) for pos in self._positions)
            update = True
        if self._volt_pos and self._volt_pos in data:
            update = True
            if self._voltage_to_block:
                self.block.info_values[INFO_VALUE_VOLTAGE] = \
                    round(data[self._volt_pos], 2)
            else:
                self.info_values[INFO_VALUE_VOLTAGE] = \
                    round(data[self._volt_pos], 2)
        if self._pf_pos and self._pf_pos in data:
            update = True
            self.info_values[INFO_VALUE_POWER_FACTOR] = \
                round(data[self._pf_pos], 2)
        if self._current_pos and self._current_pos in data:
            update = True
            self.info_values[INFO_VALUE_CURRENT] = \
                round(data[self._current_pos], 2)
        if self._tot_pos and self._tot_pos in data:
            update = True
            self.info_values[INFO_VALUE_TOTAL_CONSUMPTION] = \
                round(data[self._tot_pos]/60)
        if update:
            self._update(self.state, info_values=self.info_values)


# -*- coding: utf-8 -*-

from .device import Device

from .const import (
    LOGGER,
    STATUS_RESPONSE_LIGHTS,
    STATUS_RESPONSE_LIGHTS_STATE,
    STATUS_RESPONSE_LIGHTS_BRIGHTNESS,
    STATUS_RESPONSE_INPUTS,
    STATUS_RESPONSE_INPUTS_INPUT,
    INFO_VALUE_SWITCH,
    STATUS_RESPONSE_METERS,
    STATUS_RESPONSE_METERS_POWER,
    INFO_VALUE_CURRENT_CONSUMPTION
)

class Dimmer(Device):
    def __init__(self, block, state_pos, dim_pos):
        super(Dimmer, self).__init__(block)
        self.id = block.id
        self.device_type = "DIMMER"
        self.url = "/light/0"
        self.state = None
        self.brightness = None
        self.state_pos = state_pos
        self.dim_pos = dim_pos
        self.info_values = {}

    def update(self, data):
        new_state = data.get(self.state_pos) == 1
        self.brightness = data.get(self.dim_pos)
        values = {'brightness': self.brightness}
        for idx in range(0, 2):
            value = data.get(131 + idx*10)
            if value is not None:
                self.info_values[INFO_VALUE_SWITCH + "_" + str(idx+1)] = value > 0
        #todo, read consumption when firmware fixed
        self._update(new_state, values, None, self.info_values)

    def update_status_information(self, status):
        """Update the status information."""
        new_state = None
        lights = status.get(STATUS_RESPONSE_LIGHTS)
        values = {}
        if lights:
            light = lights[0]
            new_state = light.get(STATUS_RESPONSE_LIGHTS_STATE, None)
            self.brightness = \
                light.get(STATUS_RESPONSE_LIGHTS_BRIGHTNESS, None)
            values['brightness'] = self.brightness
        inputs = status.get(STATUS_RESPONSE_INPUTS)
        if inputs:
            for idx in range(0, 2):
                value = inputs[idx]
                if value.get(STATUS_RESPONSE_INPUTS_INPUT) is not None:
                    self.info_values[INFO_VALUE_SWITCH + "_" + str(idx+1)] = \
                        value.get(STATUS_RESPONSE_INPUTS_INPUT) > 0
        meters = status.get(STATUS_RESPONSE_METERS)
        if meters and len(meters) > 0:
            meter = meters[0]
            if meter.get(STATUS_RESPONSE_METERS_POWER) is not None:
                self.info_values[INFO_VALUE_CURRENT_CONSUMPTION] = \
                    round(float(meter.get(STATUS_RESPONSE_METERS_POWER)))
        self._update(new_state, values, None, self.info_values)

    def _send_data(self, state, brightness=None):
        url = self.url + "?"
        if state is not None:
            if not state or brightness == 0:
                url += "turn=off"
                self._send_command(url)
                return
            url += "turn=on&"
        if brightness is not None:
            url += "brightness=" + str(brightness) + "&"

        self._send_command(url)

    def turn_on(self, brightness=None):
        self._send_data(True, brightness)

    def turn_off(self):
        self._send_data(False)

    def get_dim_value(self):
        return self.brightness

    def set_dim_value(self, value):
        self._send_data(True, value)
